#include "orbit.hpp"

#include "glm/gtx/quaternion.hpp"

Orbit::Orbit(int vertexCount, glm::vec3 up)
{
    const float pi = 3.14159265359;
    const glm::vec3 worldUp = glm::vec3(0.0, 1.0, 0.0);

    // Compute the rotation between world 'up' and our defined up
    glm::quat rot = glm::rotation(worldUp, normalize(up));

    for (int i = 0; i < vertexCount; i++)
    {
        float a = (float)i / (float)vertexCount;
        a *= 2.0 * pi;

        glm::vec3 v = glm::vec3(cos(a), 0.0, sin(a));

        // Rotate generated point to align with new up vector
        v = rot * v;

        _vertices.push_back(v.x);
        _vertices.push_back(v.y);
        _vertices.push_back(v.z);
    }

    glGenVertexArrays(1, &_vao);
    glGenBuffers(1, &_vbo);

    glBindVertexArray(_vao);

    glBindBuffer(GL_ARRAY_BUFFER, _vbo);
    size_t vboBufferSize = _vertices.size() * sizeof(float);
    glBufferData(GL_ARRAY_BUFFER, vboBufferSize, &_vertices[0], GL_STATIC_DRAW);

    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);
}

void Orbit::render()
{
    glBindVertexArray(_vao);
    glBindBuffer(GL_ARRAY_BUFFER, _vbo);

    glDrawArrays(GL_LINE_LOOP, 0, _vertices.size() / 3);
}

Orbit::~Orbit()
{
    glDeleteVertexArrays(1, &_vao);
    glDeleteBuffers(1, &_vbo);
}

